import sys

def call(func, path, *args, **kwargs):
  if path == "-":
    func(sys.stdout, *args, **kwargs)
  elif path is None:
    pass
  else:
    with open(path, "w") as f:
      func(f, *args, **kwargs)

def balancesheet(stream, cur):
  cur.execute("""
    SELECT
      entry.type,
      entry.name,
      SUM(journal.amount)
    FROM journal RIGHT JOIN entry ON journal.entry=entry.id
    WHERE entry.type in ("A","D","N")
    GROUP BY entry.name;
  """)
  for row in cur:
    print(_recover_entry_name(row[0], row[1]), row[2], file=stream, sep=",")

def plstatement(stream, cur, begin=None, end=None):
  if begin is None:
    begin = "0000-00-00";
  if end is None:
    end = "9999-12-31";

  cur.execute("""
    SELECT
      entry.type,
      entry.name,
      SUM(journal.amount)
    FROM journal
      RIGHT JOIN entry ON journal.entry=entry.id
      RIGHT JOIN tx    ON journal.tx   =tx.id
    WHERE
      entry.type in ("E","R") AND tx.date BETWEEN ? AND ?
    GROUP BY entry.name;
  """, (begin, end))
  for row in cur:
    print(_recover_entry_name(row[0], row[1]), row[2], file=stream, sep=",")

def balancetransition(stream, cur):
  cur.execute("""
    SELECT
      date,
      SUM(sum)
        OVER(ORDER BY date ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)
    FROM (
      SELECT
        tx.date AS date,
        SUM(
          CASE WHEN entry.type="A" THEN journal.amount ELSE -journal.amount END
        ) AS sum
      FROM journal
        RIGHT JOIN entry ON journal.entry=entry.id
        RIGHT JOIN tx    ON journal.tx   =tx.id
      WHERE    entry.type in ("A","D","N")
      GROUP BY tx.date
    );
  """)
  for row in cur:
    print(*row, file=stream, sep=",")

def pltransition(stream, cur, datefmt=None):
  if datefmt is None:
    datefmt = "%Y-%m-%d"
  cur.execute("""
    WITH temp AS (
        SELECT
          STRFTIME(?, tx.date) AS period,
          entry.type           AS type,
          SUM(amount)          AS sum
        FROM journal
          RIGHT JOIN entry ON journal.entry=entry.id
          RIGHT JOIN tx    ON journal.tx   =tx.id
        WHERE    entry.type in ("E","R")
        GROUP BY period, entry.type
      )
    SELECT
      a.period AS period,
      a.sum    AS expense,
      IFNULL(b.sum, 0) AS revenue
    FROM temp AS a LEFT JOIN temp AS b ON a.period=b.period AND b.type="R"
    WHERE a.type="E";
  """, [datefmt])
  for row in cur:
    print(*row, file=stream, sep=",")

def _recover_entry_name(t: str, name: str):
  m = {
    "A": "asset",
    "D": "debt",
    "N": "net",
    "E": "expense",
    "R": "revenue",
  }
  return f"{m[t]}/{name}"
